/*
 *  GUI/EntriesList/EntriesList.cs
 *  Copyright (C) 2006-2007 Marcin Krystian Krzywonos
 *  License: GNU/GPL version 2
 */



using System;
using System.Collections;
using Incollector.Engine;


namespace Incollector.GUI.Widgets {

	public abstract class EntriesList : Gtk.VBox {
		
		// Widgets connected with EntriesList
		public new Gtk.Window       ParentWindow;
		public Widgets.EntryPreview PreviewWidget;
		public SearchEntry          SearchEntryWidget;
		public Widgets.TagsList     TagsListWidget;
		
		// Parameters
		public abstract int         NSelectedEntries { get ; }
		public abstract int         NEntries         { get ; }
		public abstract Gtk.Widget  FocusWidget      { get ; }
		
		// Signals
		public delegate void SelectionChangedHandler (int NItems);
		public SelectionChangedHandler SelectionChanged;
		
		
		// Other functions
		public abstract void ReloadEntries ();
		public abstract void Clear ();
		
		public void LoadPreview ()
		{
			LoadPreview (false);
		}
		
		public abstract void LoadPreview (bool ForceLoad);
		
		public abstract ArrayList GetSelectedEntries ();
		public abstract int       GetDroppedEntryID  (Gtk.DragDataReceivedArgs args);
		
		
		public void AddEntry ()
		{
			this.AddEntry ("", "");
		}
		
		public void AddEntry (string DataTypeName)
		{
			this.AddEntry ("", DataTypeName);
		}
		
		
		public abstract void AddEntry (string Content, string DataTypeName);
		public abstract void RemoveSelectedEntries ();
		public abstract void ModifySelectedEntry ();
	}
}
