/*
 *  LabelButton.cs
 *  Copyright (C) 2006-2007 Marcin Krystian Krzywonos
 *  License: GNU/GPL, version 2
 */


using System;
using GLib;
using Gtk;
using Gdk;
using Mono.Unix;


namespace Incollector.GUI.Widgets {
	
	public class LabelButton : Gtk.HBox {
		
		protected Gtk.Label label;
		protected Gtk.Arrow arrow;
		protected Gdk.Color normalColor = new Gdk.Color (16, 16, 239);
		protected Gdk.Color hoverColor = new Gdk.Color (96, 128, 255);
		
		
		public float Xalign {
			get { return label.Xalign;  }
			set { label.Xalign = value; }
		}
		
		public float Yalign {
			get { return label.Yalign;  }
			set { label.Yalign = value; }
		}
		
		public Gdk.Color NormalColor {
			get { return this.normalColor;  }
			set {
				label.ModifyFg (Gtk.StateType.Normal, value);
				this.normalColor = value;
			}
		}
		
		public Gdk.Color HoverColor {
			get { return this.hoverColor;  }
			set { this.hoverColor = value; }
		}

		public Gtk.ArrowType ArrowType {
			get { return this.arrow.ArrowType; }
			set { this.arrow.ArrowType = value; }
		}
		
		
		public delegate void ClickHandler ();
		public event ClickHandler Clicked;
		
		
		public LabelButton (string Text, Gtk.ArrowType arrow)
		{
			CreateLabelButton (Text, arrow);
		}
		
		
		public void CreateLabelButton (string Text, Gtk.ArrowType arrow_type)
		{
			Gtk.EventBox evbox = new Gtk.EventBox ();
			evbox.VisibleWindow = false;
			evbox.ButtonReleaseEvent += new ButtonReleaseEventHandler (on_Clicked);
			evbox.EnterNotifyEvent += new EnterNotifyEventHandler (on_Enter);
			evbox.LeaveNotifyEvent += new LeaveNotifyEventHandler (on_Leave);
			this.PackStart (evbox, false, false, 0);
			
			Gtk.HBox box = new Gtk.HBox ();
			box.Spacing = 6;
			evbox.Add (box);
			
			arrow = new Gtk.Arrow (arrow_type, Gtk.ShadowType.In);
			box.PackStart (arrow, false, false, 0);
			
			label = new Label (Text);
			label.ModifyFg (Gtk.StateType.Normal, this.normalColor);
			label.UseUnderline = true;
			label.UseMarkup = true;
			label.Markup   = Text;
			label.Xalign = 0;
			label.Yalign = (float) 0.50;
			box.PackStart (label, false, false, 0);
			
			
			this.ShowAll ();
		}
		
		
		protected void on_Enter (object o, EnterNotifyEventArgs args)
		{
			label.ModifyFg (Gtk.StateType.Normal, this.hoverColor);
			label.GdkWindow.Cursor = new Gdk.Cursor (Gdk.CursorType.Hand2);
		}
		
		
		protected void on_Leave (object o, LeaveNotifyEventArgs args)
		{
			label.ModifyFg (Gtk.StateType.Normal, this.normalColor);
			label.GdkWindow.Cursor = new Gdk.Cursor (Gdk.CursorType.TopLeftArrow);
		}
		
		
		protected void on_Clicked (object o, ButtonReleaseEventArgs args)
		{
			if ((args.Event.X != 0) && ((args.Event.Y != 0)) && (args.Event.Button == 1) && (this.Clicked != null))
				Clicked ();
		}
	}
}
