/*
 *  src/Searching/Queries/Category.cs
 *  Copyright (C) 2007 by Marcin Krystian Krzywonos
 *  License: GNU/GPL version 2
 */


using System;
using System.Xml;
using Gtk;
using Mono.Unix;
using Incollector.DataTypes;


namespace Incollector.Searching.Queries {
	
	public class Category : Query {
		
		protected Gtk.Widget   queryWidget;
		protected Gtk.ComboBox categoriesListWidget;
		protected Gtk.ComboBox conditionalsListWidget;
		
		public enum ConditionalEnum {
			Is    = 0,
			IsNot = 1
		}
		
		public string CategoryValue {
			get { return DataTypes.DataTypes.DataTypesList [categoriesListWidget.Active].Name; }
			set { categoriesListWidget.Active = DataTypes.DataTypes.GetDataTypeID (value); }
		}
		
		public ConditionalEnum Conditional {
			get { return (ConditionalEnum) this.conditionalsListWidget.Active; }
			set { this.conditionalsListWidget.Active = (int) value; }
		}
		
		public override Gtk.Widget QueryWidget {
			get { return queryWidget; }
		}
		
		public override string ID {
			get { return "category"; }
		}
		
		public override string Name {
			get { return Catalog.GetString ("Category"); }
		}
		
		
		public Category ()
		{
			queryWidget = new Gtk.HBox ();
			(queryWidget as Gtk.HBox).Spacing = 6;
			
			conditionalsListWidget = new Gtk.ComboBox (new string [] {
								Catalog.GetString ("is"),
								Catalog.GetString ("is not")
							});
			conditionalsListWidget.Active = 0;
			
			categoriesListWidget = new Gtk.ComboBox (new string [] {});
			
			foreach (DataType dt in DataTypes.DataTypes.DataTypesList)
				categoriesListWidget.AppendText (dt.Title);
			
			categoriesListWidget.Active = 0;
			
			
			(queryWidget as Gtk.HBox).PackStart (conditionalsListWidget, true, true, 0);
			(queryWidget as Gtk.HBox).PackStart (categoriesListWidget, true, true, 0);
			
			queryWidget.ShowAll ();
		}
		
		
		public override Query New ()
		{
			return (Query) new Category ();
		}
		
		
		public override Query Clone ()
		{
			Category query = new Category ();
			query.CategoryValue = this.CategoryValue;
			query.Conditional = this.Conditional;
			
			return query;
		}
		
		
		public override bool CheckQuery (Engine.Entry entry)
		{
			switch (Conditional) {
				case ConditionalEnum.Is:
					return String.Compare (entry.Data.Name, CategoryValue) == 0;
				
				case ConditionalEnum.IsNot:
					return String.Compare (entry.Data.Name, CategoryValue) != 0;
			}
			
			return true;
		}

		
		public override Query ReadXml (XmlTextReader reader)
		{
			Category query = new Category ();
			
			query.Conditional = (ConditionalEnum) System.Enum.Parse (typeof (ConditionalEnum), reader.GetAttribute ("conditional"));
			query.CategoryValue = reader.GetAttribute ("category");
			
			return query;
		}
		
		
		public override void WriteXml (XmlTextWriter writer)
		{
			writer.WriteStartElement (this.ID);
				writer.WriteAttributeString ("conditional", Conditional.ToString ());
				writer.WriteAttributeString ("category", CategoryValue);
			writer.WriteEndElement ();
		}
	}
}
