/*
 *  Searching/SearchFolders.cs
 *  Copyright (C) 2006-2007 Marcin Krystian Krzywonos
 *  License: GNU/GPL version 2
 */


using System;
using System.Collections;
using System.IO;
using System.Xml;
using Incollector.Engine;


namespace Incollector.Searching {
	
	public static class SearchFolders {
		
		public static ArrayList QueriesList = new ArrayList (new object [] {
							(Query) new Queries.Tags (),
							(Query) new Queries.Rating (),
							(Query) new Queries.Category (),
							(Query) new Queries.Comment (),
							(Query) new Queries.Content (),
						});
		
		
		public static SearchFolder ActiveFolder {
			get { return activeFolder; }
		}
		
		
		private static ArrayList    data   = new ArrayList ();
		public  static int          LastID = -2;
		private static SearchFolder activeFolder = new SearchFolder (-2); 
		private static bool         saving_now;
		
		
		public delegate void AddedHandler     (SearchFolder folder);
		public delegate void ModifiedHandler  (SearchFolder folder);
		public delegate void RemovedHandler   (SearchFolder folder);
		public delegate void ActivatedHandler (SearchFolder folder);
		
		public static event AddedHandler     Added;
		public static event ModifiedHandler  Modified;
		public static event RemovedHandler   Removed;
		public static event ActivatedHandler Activated;
		
		
		public static bool Load ()
		{
			data = new ArrayList ();
			SearchFolders.Activate (new SearchFolder (-1));
			LastID = 0;
			
			StreamReader reader;
			
			try {
				reader = new StreamReader (FilesPaths.SearchFoldersFile, System.Text.Encoding.UTF8);
			} catch  {
				return false;
			}
			
			XmlTextReader read           = new XmlTextReader (reader);
			bool          ReadingQueries = false;
			SearchFolder  folder         = new SearchFolder ();
			
			try {
				while (read.Read ()) {
					if ((read.NodeType == XmlNodeType.Element) || (read.NodeType == XmlNodeType.EndElement)) {
						if ((read.Name == "folder") && (read.NodeType == XmlNodeType.Element)) {
							folder = new SearchFolder (-1);
							folder.ID = Convert.ToInt32 (read.GetAttribute ("id"));
							folder.Name = read.GetAttribute ("name");
							folder.SearchOperator = (LogicalOperator) System.Enum.Parse (typeof (LogicalOperator), read.GetAttribute ("operator"));
							
							if (LastID < folder.ID)
								LastID = folder.ID;
							
							ReadingQueries = true;
						}
						
						
						if (ReadingQueries == true) {
							foreach (Query query in SearchFolders.QueriesList) {
								if (read.Name == query.ID) {
									folder.Queries.Add (query.ReadXml (read));
									break;
								}
							}
						}
						
						
						if ((read.Name == "folder") && (read.NodeType == XmlNodeType.EndElement)) {
							ReadingQueries = false;
							folder.ID = Convert.ToInt32 (read.GetAttribute ("id"));
							folder.Save ();
						}
					}
				}
			} catch {
				reader.Close ();
				return false;
			}
			
			reader.Close ();
			return true;
		}
		
		
		public static void Save ()
		{
			if (saving_now == true)
				return;
			
			saving_now = true;
			
			XmlTextWriter f = new XmlTextWriter (FilesPaths.SearchFoldersSavingFile, System.Text.Encoding.UTF8);
			
			f.Formatting = Formatting.Indented;
			f.WriteStartDocument ();
			f.WriteStartElement (null, "SearchFolders", "");
				
				foreach (SearchFolder folder in data) {
					if (folder.Special)
						continue;
					
					f.WriteStartElement ("folder");
					f.WriteAttributeString ("id", folder.ID.ToString ());
					f.WriteAttributeString ("name", folder.Name);
					f.WriteAttributeString ("operator", folder.SearchOperator.ToString ());
					
						foreach (Query query in folder.Queries)
							query.WriteXml (f);
					
					f.WriteEndElement ();
				}
			f.WriteEndElement ();
			
			f.Close ();
			
			
			
			try {
				File.Delete(FilesPaths.SearchFoldersFile);
			} finally {
				File.Move(FilesPaths.SearchFoldersSavingFile, FilesPaths.SearchFoldersFile);
			}
			
			saving_now = false;
		}
		
		
		public static void Add (SearchFolder folder)
		{
			data.Add (folder);
			
			if (Added != null)
				Added (folder);
		}
		
		
		public static void Remove (SearchFolder folder)
		{
			data.Remove (folder);
			
			if (Removed != null)
				Removed (folder);
			
			if (SearchFolders.ActiveFolder.ID == folder.ID)
				Activate (new SearchFolder (-1));
		}
		
		
		public static SearchFolder GetFolder (int id)
		{
			foreach (SearchFolder folder in data)
				if (folder.ID == id)
					return folder;
			
			return new SearchFolder ();
		}
		
		
		public static void Replace (SearchFolder folder)
		{
			for (int i = 0; i < data.Count; i++)
				if (((SearchFolder) data [i]).ID == folder.ID) {
					data [i] = folder;
					
					if (activeFolder.ID == folder.ID)
						activeFolder = folder;
					
					if (Modified != null) {
						Modified (folder);
					}
					
					return;
				} 
		}
		
		
		public static void Activate (SearchFolder folder)
		{
			if (activeFolder.ID == folder.ID)
				return;
			
			activeFolder = folder;
			
			if (Activated != null)
				Activated (activeFolder);
		}
	}
}
